/* Generated automatically. */
static const char configuration_arguments[] = "/pobj/msp430-gcc-4.6.3/gcc-4.6.3/configure --enable-languages=c,c++ --enable-cpp --with-gmp=/usr/local --disable-lto --target=msp430 --disable-nls --disable-shared --prefix=/usr/local --sysconfdir=/etc --mandir=/usr/local/man --infodir=/usr/local/info --localstatedir=/var --disable-silent-rules --disable-gtk-doc";
static const char thread_model[] = "single";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { NULL, NULL} };
